library ieee;
use ieee.std_logic_1164.all;
use IEEE.numeric_std.all;

entity nexys4_vga_minimal is Port(
	clk : in std_logic;
    vgaRed, vgaBlue, vgaGreen : out std_logic_vector(3 downto 0);
    Hsync, Vsync : out std_logic);
end nexys4_vga_minimal;

architecture Behavioral of nexys4_vga_minimal is

component clk_wiz_0 is port(
	clk_in : in std_logic;
	clk_out : out std_logic);
end component;

component vga_640 is port(
	clk_vga: in std_logic;
	horizontal_counter : out std_logic_vector (9 downto 0);
	vertical_counter   : out std_logic_vector (9 downto 0);
    hs_out   : out std_logic;
    vs_out   : out std_logic);
end component;

signal clk25, vga_rect, vga_visible_v, vga_visible_h: std_logic:='0';
signal hc_v, vc_v: std_logic_vector (9 downto 0):=(others => '0');
signal screen_h: integer range 0 to 1023:=0;
signal screen_v: integer range 0 to 511:=0;

begin

pll0 : clk_wiz_0 port map(
	clk_in => clk,
	clk_out => clk25);
	
vga0 : vga_640 port map(
    clk_vga => clk25,
    horizontal_counter => hc_v,
    vertical_counter => vc_v,
    hs_out => Hsync,
    vs_out => Vsync);
	
vgaRed <= (others => '0') when vga_rect = '0' else (others => '1');
vgaGreen <= (others => '0') when vga_rect = '0' else (others => '1');
vgaBlue <= (others => '0') when vga_rect = '0' else (others => '1');

process begin
	wait until rising_edge(clk25);
	vga_visible_h <= '0';
	vga_visible_v <= '0';

	if to_integer(unsigned(hc_v)) > 146 and to_integer(unsigned(hc_v)) < 786 then
		screen_h <= to_integer(unsigned(hc_v)) - 146;
		vga_visible_h <= '1';
	else
		screen_h <= 0;
	end if;

	if to_integer(unsigned(vc_v)) > 34 and to_integer(unsigned(vc_v)) < 515 then
		screen_v <= 514 - to_integer(unsigned(vc_v));
		vga_visible_v <= '1';
	else
		screen_v <= 0;
	end if;
	
	vga_rect <= '0';
	if screen_v > 200 and screen_v < 300 and screen_h > 300 and screen_h < 400 then
		vga_rect <= '1';
	end if;
	
end process;

end Behavioral;
